#include <iostream>
#include <set>

#include "definitions.h"
#include "init_stuff.h"
#include "print_stuff.h"
#include "CartSk.h"

#include <boost/graph/graph_utility.hpp>
#include <boost/property_map/property_map.hpp>
//#include <boost/property_map.hpp>



/*********************CartSk due to hammack************************/
void CartSkeleton(myGraph& G){

	vector< set<int> > N(num_vertices(G));
	init_neighborhood(G, N);
	
	/*myEdgeW weightG = get(edge_weight, G);*/
	eIt ei, ei_end;
	for(tie(ei,ei_end) = edges(G); ei!=ei_end; ei++){
		int x = target(*ei, G);
	    int y = source(*ei, G);
	/*	cout << endl << "N" << x << "="; 
		showSet(N[x]);
		cout << endl << "N" << y << "="; 
		showSet(N[y]);*/
		
		set<int> N_inters; 
		init_intersection(N[x], N[y],N_inters);
	    
		myEdgeW weightG = get(edge_weight, G);
	
		/**exists z \in edgeN s.t. N[x] \intersection N[y] is proper subset of N[x] \intersection N[z] 
				         and N[x] \intersection N[y] is proper subset of N[y] \intersection N[z] */
		if(condi1(N[x],N[y],N_inters, x,y, N)){
			put(weightG, *ei, 0); 
			//cout << endl << *ei << " is nonCart";
			}
		/**exists z in edgeN s.t. N_x \proper_subset N_z \proper_subset N_y
					or  N_y \proper_subset N_z \proper_subset N_x*/
		else if(condi2(N[x],N[y],N_inters, x,y, N)){
			//cout << endl << *ei << " is nonCart";
			put(weightG, *ei, 0); 
		}
		else{ 
			//cout << endl << *ei <<" marked as Cartesian ";
			put(weightG, *ei, 1); 
		}
	
	
	}
	
}



bool condi1(set<int>& N1, set<int>& N2,set<int>& z_Set, int x, int y, vector< set<int> >& N){
	
	set<int> inters_Nxy;
	init_intersection(N1,N2,inters_Nxy);
	
	set<int>::iterator si;
	for(si=z_Set.begin();si!=z_Set.end();si++){
		if(*si!=x && *si!=y){

			set<int> inters_Nxz;
			set<int> inters_Nyz;
			init_intersection(N1,N[*si],inters_Nxz);
			init_intersection(N2,N[*si],inters_Nyz);
			
			
			
			if(is_proper_subset(inters_Nxy,inters_Nxz) && is_proper_subset(inters_Nxy,inters_Nyz)){
				//cout << endl << "cond1 for (" << x << "," << y << ") found z = "  << *si;
				//cout << endl << "N" << *si << "="; 
				//showSet(N[*si]);
	
				return true;
			}
		}
	}
	 
	return false;
} 




bool condi2(set<int>& N1, set<int>& N2,set<int>& z_Set, int x, int y, vector< set<int> >& N){
	
	set<int> N_A,N_B;/**N_A \proper subset of N_B*/
	N_A.clear();N_B.clear();
	if(is_proper_subset(N1,N2)){
		N_A = N1;
		N_B = N2;
	}
	else if(is_proper_subset(N2,N1)){
		N_A = N2;
		N_B = N1;
	}
	else return false;
	
	set<int>::iterator si;
	for(si=z_Set.begin();si!=z_Set.end();si++){
		if(*si!=x && *si!=y){
			
			if(is_proper_subset(N_A,N[*si]) && is_proper_subset(N[*si],N_B)){
				//cout << endl << "cond2 for (" << x << "," << y << ") found z = "  << *si;
				//cout << endl << "N" << *si << "="; 
				//showSet(N[*si]);
			
				return true;
			}
		}
	
	}
	
	return false;
}




void CartSkeleton_Feigenbaum(myGraph& G){

	//cout << endl << "init Cart_sk" << endl;
	
	vector< set<int> > N(num_vertices(G));
	init_neighborhood(G, N);
	
	set < set<int> > Pot;
	vector< set<int> > P(num_vertices(G));
	vector< set<int> > Q(num_vertices(G));
	
	init_P_Pot_Q(Pot,P,Q,G,N);
	//showMultiSet(Pot);

	vector<int> X;
	vector < vector<int> > QminusP(num_vertices(G));
	vector< set< set<int> > > J(num_vertices(G));
	
	property_map < myGraph, edge_weight_t >::type w = get(edge_weight, G);
    int weights[1] = {1};
    int *wp = weights;
	
	/**********************  M1  ********************************************/
	while(exits_x(P,Q,X,QminusP,G)){
		
		for(unsigned int i = 0; i<X.size();i++){
			//cout << endl <<X[i];
			for(unsigned int j = 0; j<QminusP[X[i]].size();j++){
				int y = QminusP[X[i]][j];
				J[X[i]].insert(N[y]);								
			}
			
			for(unsigned int j = 0; j<QminusP[X[i]].size();j++){
				int y = QminusP[X[i]][j];
				if(is_maximal_set(N[y] , J[X[i]])){
					//cout << endl << "mark " << X[i] << " - " << y;
					w[edge(X[i],y,G).first] = *wp;
					/**P=Px_vereinigt_Py .. delete Px,Py aus Pot, fuege ein P in Pot*/
					join_Px_Py(Pot,P[X[i]],P[y],P,X[i],y);
				}
			}
		}
	}	
	/**********************end  M1  ********************************************/
	//cout << endl << "Pot = { " ; showMultiSet(Pot); cout << " } " << endl;
	
	vector< vector< set<int> > > Ixy(num_vertices(G));
	//vector< vector<int> > ForWhich_y_Ixy_nonEmpty(num_vertices(G));
	
	if(Pot.size() > 1) init_Ixy(Ixy, N, G);
	
	/**********************  M2  ********************************************/
	
	vector< multiset< set<int> > > multi_J(num_vertices(G));
	
	while(Pot.size() > 1){
		
		/**********************  M3  ********************************************/
		vIt vi, vi_end;
		vector< vector<int> > marked(num_vertices(G));
		for(tie(vi,vi_end) =vertices(G); vi!=vi_end; vi++){
			//cout <<  endl << "**********************check x= " << *vi;
			//cout << endl << "N" << *vi << " = "; showSet(N[*vi]);
			
			marked[*vi].clear();
			//J[*vi].clear();
			multi_J[*vi].clear();
			
			vector<int> which_y;
			init_Jx(multi_J[*vi],/*J[*vi],*/Ixy[*vi],P[*vi], which_y ,G);
			if(!multi_J[*vi].empty()){
				for(unsigned int i=0; i<which_y.size();i++){
					int y = which_y[i]; //cout << endl << "y= " << y;	
					
			/**!!!!in J[*vi] is Ixy[*vi][y]  noch drin, daher wird nie strictly max erreicht,
			 thus for testing if it is strictly max, we have to delete it, thus need a MultiSet*/					
					if(is_strictly_maximal_set_inMultiSet(Ixy[*vi][y],multi_J[*vi]) && edge(*vi,y,G).second ){
			//cout<<endl<<"(str) mark "<<*vi<<"-"<< y;	
			//cout << "exists? " << edg(*vi,y,G).second;					
						w[edge(*vi,y,G).first] = *wp;
						marked[*vi].push_back(y);
					}
			/**condition: for all z \notin P[*vi] with I_xy=I_xz holds N[z] \not\subset(proper) N[y]*/
			/**sometimes condition fulfilled, although edge does not exist -- correct implementaion?*/
					else if(is_maximal_set_inMultiSet(Ixy[*vi][y],multi_J[*vi]) &&
					        condition(P[*vi],Ixy[*vi], which_y, N, y) && edge(*vi,y,G).second){
	 //cout << endl << "(nonstr) mark " << *vi << "-" << y; //cout << "exists? " << edge(*vi,y,G).second;

						w[edge(*vi,y,G).first] = *wp;
						marked[*vi].push_back(y);
					}
				}
			}
		}
		/**********************  end M3  ********************************************/
		
		for(tie(vi,vi_end) =vertices(G); vi!=vi_end; vi++)
			for(unsigned int i=0; i<marked[*vi].size(); i++)
				join_Px_Py(Pot,P[*vi],P[marked[*vi][i]],P,*vi,marked[*vi][i]);
		//cout << endl << "Pot = { " ; showMultiSet(Pot); cout << " } " << endl;
	
	}
	
	/**********************end  M2  ********************************************/
	//cout << endl << "END init Cart_sk" << endl;
}



void init_P_Pot_Q(set < set<int> >& Pot, vector< set<int> >& P, 
				  vector< set<int> >& Q, myGraph& G, vector< set<int> >& N){
	
	vIt vi, vi_end;
	aIt ai, ai_end;
	
	for(tie(vi,vi_end) = vertices(G); vi!=vi_end; vi++){
		P[*vi].insert(*vi);
		Pot.insert(P[*vi]);
		for(tie(ai,ai_end) = adjacent_vertices(*vi,G);ai!=ai_end;ai++){
			if(is_proper_subset(N[*ai],N[*vi])){
				//cout << endl << *ai << " subset of " << *vi;
				Q[*vi].insert(*ai);
			} 
		}
		/** if for y = adj_vertices(vi,G)  if N[y] is subset of N[vi]  add y in Q(vi)*/
	}

}

set<int> Qx_minus_Px(int x, set<int> P, set<int> Q, vector<int>& QminusP){
	
	set<int> DIFF;
	set_difference(Q.begin(), Q.end(),
    		       P.begin(), P.end(),
				   inserter(DIFF, DIFF.begin()));
	
	set<int>::iterator its;
	QminusP.clear();
	for (its=DIFF.begin();its!=DIFF.end();++its)
		QminusP.push_back(*its);
	
	//cout << endl << "Q-P " << x << " = ";  showSet(DIFF);
	return DIFF;			   
	
}

bool exits_x(vector< set<int> >& P,vector< set<int> >& Q, vector<int>& X, 
             vector< vector<int> >& QminusP, myGraph& G){
	
	X.clear();
	bool exist=false;
	vIt vi, vi_end;
	for(tie(vi,vi_end) =vertices(G); vi!=vi_end; vi++){
		if(!Qx_minus_Px(*vi,P[*vi],Q[*vi],QminusP[*vi]).empty()){ 
			X.push_back(*vi);
			exist=true;
		}
	}
	
	return exist;
	
}

void join_Px_Py(set< set<int> >& Pot,set<int>& set_Px,
				set<int>& set_Py, vector< set<int> >& P, int x, int y){
	
	
	set< set<int> >::iterator Px,Py;
	Px = Pot.find(set_Px);
	Py = Pot.find(set_Py);
	
	set<int> P_new; P_new.clear();
	set_union(set_Px.begin(), set_Px.end(),
			  set_Py.begin(), set_Py.end(),
			  inserter(P_new, P_new.begin()));
	
	/*cout << endl << "Pot = "; showMultiSet(Pot);
	cout << endl << "p" <<x<< " = "; showSet(set_Px);
	cout << endl << "p" <<y<<" = "; showSet(set_Py);
	cout << endl << "union = "; showSet(P_new);
	cout << endl;*/

	Pot.erase(Px);
	if(Px!=Py) Pot.erase(Py);
	Pot.insert(P_new);
	set<int>::iterator q;
	
	
	for(q = P_new.begin(); q != P_new.end(); ++q){
		set<int>::iterator q2;
		for(q2 = P_new.begin(); q2 != P_new.end(); ++q2)
			P[*q].insert(*q2);
	}
	
	/*for(q = P_new.begin(); q != P_new.end(); ++q){
		cout << endl << "p" <<*q<< " = "; showSet(P[*q]);
	}*/
	
}

void init_Ixy(vector< vector< set<int> > >&  Ixy, vector< set<int> >&  N, myGraph& G){
   
	vIt vi, vi_end, vi2, vi2_end;
	
	for(tie(vi,vi_end) =vertices(G); vi!=vi_end; vi++){
		Ixy[*vi].resize(num_vertices(G));
		for(tie(vi2,vi2_end) =vertices(G); vi2!=vi2_end; vi2++){
			set_intersection(N[*vi].begin(), N[*vi].end(),
							 N[*vi2].begin(), N[*vi2].end(),
							 inserter(Ixy[*vi][*vi2], Ixy[*vi][*vi2].begin()));
		}
	}
   
}

void init_Jx(multiset< set<int> >& Jx, /*set< set<int> >& Jx, */ vector< set<int> >& Ix,
			 set<int> & Px, vector<int>& which_y,  myGraph& G){
	
	vIt vi, vi_end;
	which_y.clear();
	for(tie(vi,vi_end) =vertices(G); vi!=vi_end; vi++)
		if(!Ix[*vi].empty() && Px.find(*vi)==Px.end() ){
			Jx.insert(Ix[*vi]);
			//multi_Jx.insert(Ix[*vi]);
			which_y.push_back(*vi);
			
		}
}

bool condition(set<int> & Px, vector< set<int> >& Ix, vector<int>& which_y, vector< set<int> >&  N, int y){
	
	//cout << endl << "general y = " << y;
	//cout << endl << "N" << y << " = "; showSet(N[y]);
	//cout << endl << "I_x," << y << " ==? "; showSet(Ix[y]);
	for(unsigned int i=0; i<which_y.size();i++){
		int z = which_y[i];
		if(z!=y){
			if(is_subset(Ix[y], Ix[z]) && is_subset(Ix[z], Ix[y])){
		
		//		cout << endl << "N" << z << " = "; showSet(N[z]);
		//		cout << endl << "I_x," << z << " = "; showSet(Ix[z]);
				if(is_subset(N[z], N[y] )){
					return false;
				}
			}
		}
		
	}
	
	return true;

}


//void HammackApproach(myGraph& G, eD edge, vector< set<int> >& N){
void init_cart_skeleton_graph(myGraph &G, myGraph &CartSk, myGraph &CartSk_edgeID){
	
	eIt ei, ei_end;
	property_map < myGraph, edge_weight_t >::type w = get(edge_weight, G);
	myEdgeID edge_id=get(edge_index,G);
	int edgeID=0;
    
	for (tie(ei, ei_end) = edges(G); ei != ei_end; ++ei){
		  if(w[edge(source(*ei,G),target(*ei,G),G).first]==1){
			add_edge(source(*ei, G),target(*ei, G),get(edge_id, *ei),CartSk_edgeID);
		 	add_edge(source(*ei, G),target(*ei, G),edgeID,CartSk);
		 	++edgeID;
		 }
	}

}

