#include <iostream>
#include <vector>
#include<set>
#include<math.h>

#include "definitions.h"
#include "blow_down.h"
#include "integer_pfd.h"
#include "init_stuff.h"
#include "print_stuff.h"
#include "../bfs.h"

#include <boost/config.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graph_utility.hpp>
#include <boost/graph/connected_components.hpp>
#include <boost/graph/breadth_first_search.hpp>
#include <boost/graph/copy.hpp>



using namespace std;
using namespace boost;


void assign_edID(myGraph& G){

	eIt ei, ei_end;
	myEdgeID edgeID_G = get(edge_index, G);
	int index = 0;
	for(tie(ei,ei_end) = edges(G); ei!=ei_end; ei++){
		put(edgeID_G,*ei,index);
		++index;
	}
	
}

bool check_if_CartSk_covers_G(myGraph& CartSk){
	
	vIt vi, vi_end;
	for(tie(vi,vi_end) = vertices(CartSk); vi!=vi_end;vi++){
		//cout << endl << "deg " << *vi << " = " << out_degree(*vi,CartSk);
		if(out_degree(*vi,CartSk) == 0){
			//cout << "return false";
			return false;
		}
	}
	
	return true;
}



void init_RelGraph(myGraph& G, myGraph& CartSk, myGraph& CartSk_edgeID, myGraph& CartRelGraph,
				   myGraph& ColorGraph, vector< vector<int> >& all_colors_of_edge){
	
	vector<int> componentCG(num_vertices(ColorGraph)); 
	vector<int> componentColorG(num_edges(G)); 

	
	
	/*int num_components =*/ connected_components(ColorGraph, &componentCG[0]);
	myEdgeID edgeID_CSk = get(edge_index, G);
	eIt ei, ei_end;
	for(tie(ei,ei_end) = edges(CartSk_edgeID); ei!=ei_end; ei++){
		int eID = get(edgeID_CSk,*ei);
		//cout << endl <<eID;
		//cout << endl;
		componentColorG[eID] = componentCG[all_colors_of_edge[eID][0]];
	}
	//cout << endl << num_components << " of ColorG" ;

	
	//eIt ei, ei_end;
	/**init relGraph*/
	//myEdgeID edgeID_CSk = get(edge_index, G);
	myEdgeID edgeID_C = get(edge_index, CartSk);
	for(tie(ei,ei_end) = edges(CartSk); ei!=ei_end; ei++){
		int v_s = source(*ei, CartSk);
		int v_t = target(*ei, CartSk);
		int eID = get(edgeID_CSk,edge(v_s,v_t,CartSk_edgeID).first);
		int eID2 = get(edgeID_C,*ei);
		//cout << endl <<  eID << "  " << eID2;
		eIt ei2, ei2_end;
		for(tie(ei2,ei2_end) = edges(CartSk); ei2!=ei2_end; ei2++){
			int v_s2 = source(*ei2, CartSk);
			int v_t2 = target(*ei2, CartSk);
			int eID3 = get(edgeID_CSk,edge(v_s2,v_t2,CartSk_edgeID).first);
			int eID4 = get(edgeID_C,*ei2);
//			cout << endl << eID <<" "<<eID2<< " " <<eID3<<" "<<eID4 << " " << componentColorG[eID] << " "
//				 << componentColorG[eID3] << endl;

			if((int)(num_vertices(CartRelGraph))<=eID2){
				int numV =num_vertices(CartRelGraph);
				for(;numV<=eID2;numV++)
					add_vertex(CartRelGraph);
			}

			if((int)(num_vertices(CartRelGraph))<=eID4){
				int numV =num_vertices(CartRelGraph);
				for(;numV<=eID4;numV++)
					add_vertex(CartRelGraph);
			}
			
			if(componentColorG[eID] == componentColorG[eID3] && eID != eID3 && eID2!=eID4 &&
			   !edge(eID2,eID4,CartRelGraph).second )
				add_edge(eID2,eID4,CartRelGraph);
		}
	}
}

void partial_marking_AsPossibleCartesian(myGraph& G, set<int>& S1, set<int>& S2, vector<int>& V_ID){
	
	myEdgeW weightG = get(edge_weight, G);
	
	set<int>::iterator s1;
	set<int>::iterator s2;
	for(s1 = S1.begin(); s1!=S1.end();++s1)
		for(s2 = S2.begin(); s2!=S2.end();++s2){
		eD edge_D = edge(V_ID[*s1],V_ID[*s2],G).first;
		if(get(weightG, edge_D) != 1) 
			put(weightG, edge_D, 2); /**mark edges in G as possibleCartesian, 
									   if not already as Cartesian identified, 
									   now if in addition not already determined as nonCartesian*/

	}
}

void partial_marking_asCartesian(myGraph& G, set<int>& S1, set<int>& S2, vector<int>& V_ID){
	
	myEdgeW weightG = get(edge_weight, G);
	
	set<int>::iterator s1;
	set<int>::iterator s2;
	for(s1 = S1.begin(); s1!=S1.end();++s1)
		for(s2 = S2.begin(); s2!=S2.end();++s2){
		eD edge_D = edge(V_ID[*s1],V_ID[*s2],G).first;
		put(weightG, edge_D, 1); /**mark edges in G as Cartesian*/
		
	}
}

void partial_marking_asnonCartesian(myGraph& G, set<int>& S1, set<int>& S2, vector<int>& V_ID){
	
	myEdgeW weightG = get(edge_weight, G);
	
	set<int>::iterator s1;
	set<int>::iterator s2;
	for(s1 = S1.begin(); s1!=S1.end();++s1)
		for(s2 = S2.begin(); s2!=S2.end();++s2){
			if(edge(V_ID[*s1],V_ID[*s2],G).second){
				eD edge_D = edge(V_ID[*s1],V_ID[*s2],G).first;
				put(weightG, edge_D, 0); /**mark edges in G as NONCartesian*/
			}
		
	}
}

void partial_coloring(myGraph& G, int comp, set<int>& S1, set<int>& S2, vector<int>& V_ID,
					  vector< vector<int> >& colors_of_edge, vector<int>& new_colors, myGraph& ColorG,
					  vector<bool>& proper_color_conti){
	
	//cout << endl << "in partial coloring";
	//myEdgeW weightG = get(edge_weight, G);
	myEdgeID edgeID_G = get(edge_index, G);
	
	set<int>::iterator s1;
	set<int>::iterator s2;
	for(s1 = S1.begin(); s1!=S1.end();++s1){
		for(s2 = S2.begin(); s2!=S2.end();++s2){
			eD edge_D = edge(V_ID[*s1],V_ID[*s2],G).first;
			int eID = get(edgeID_G,edge_D);
			
			for(unsigned int i=0; i<colors_of_edge[eID].size();i++)
				if(!edge(colors_of_edge[eID][i],new_colors[comp],ColorG).second){
					proper_color_conti[comp] = true;
					//8:29 PM 6/19/2009 here das wieder vlt raus
					add_edge(colors_of_edge[eID][i],new_colors[comp],ColorG);
				}
			
			colors_of_edge[eID].push_back(new_colors[comp]);
		//	cout << endl << edge_D << " ID: " << eID << " gets_color " << new_colors[comp];
		}
	}
	
	/**edgeID -- add_color+num_newcolors to color_of_edge(edgeID)*/
}

/**successively global_CartSk computed by local CartSK of each decomposed neighborhood*/
void stepwise_CartesianEdges_and_Colorgraph(myGraph& G, myGraph& locCartSk, myGraph& local_CartRelGraph,
											myGraph& N, myGraph& thinN, VecOfSets& S, vector<int>& V_ID,
											myGraph& ColorG, vector< vector<int> >& colors_of_edge,
											bool& color_conti){
	
	/*eIt ei, ei_end;
	//myEdgeID edgeID_induced = get(edge_index, induced_N);
	myEdgeW weightN = get(edge_weight, thinN);
	for(tie(ei,ei_end) = edges(thinN); ei!=ei_end; ei++){
		cout << endl << *ei ;
		//eD edge_D = edge(V_ID[source(*ei,N)],V_ID[target(*ei,N)],G).first;
		if(get(weightN, *ei) == 1) cout << " is Cartesian ";
		else cout << " is nonCartesian ";
	}*/

	//init_color_graph(G, local_CartSk, local_CartRelGraph, induced_N, thinN, S, V_ID);
	
	
	
	vector<int> component(num_vertices(local_CartRelGraph)); 
	int num_components = connected_components(local_CartRelGraph, &component[0]);
	
	vector<int> new_colors; new_colors.clear();
	int num_old_colors = num_vertices(ColorG);
	
	for(int i = 0; i<num_components; i++) add_vertex(ColorG);
	int num_new_colors = num_vertices(ColorG) - num_old_colors ;
	for(int i = num_old_colors; i<int(num_vertices(ColorG)); i++){
		new_colors.push_back(i);
	}
	
	
	vector<bool> proper_color_conti(num_new_colors);
	//cout << endl << num_new_colors <<" new_colors: ";
	for(unsigned int i = 0; i<new_colors.size();i++){
		proper_color_conti[i] = false;/**component i of RelGraph corresponds then to color new_colors[i]*/
		//cout << " " << new_colors[i];  
	}
	//cout << endl;
	
	/*vIt vi, vi_end;
	for(tie(vi,vi_end) = vertices(locCartSk); vi!=vi_end; vi++){
		cout << "\n S[" << *vi << "] = ";
		set<int>::iterator its;
		for (its=(S[*vi]).begin();its!=(S[*vi]).end();++its)
			cout<<V_ID[*its]<<" ";
		cout << "\n        ";
		for (its=(S[*vi]).begin();its!=(S[*vi]).end();++its)
			cout<<*its<<" ";
	}	*/
	

//Tue 12 Jan 2010 03:46:14 PM CET 
/*	myEdgeW weightG = get(edge_weight, G);*/
	myEdgeID edgeID_CSk = get(edge_index, locCartSk);	
	eIt ei, ei_end;
	
	vIt v1, v1_end, v2, v2_end,v3;	
	tie(v3,v2_end) = vertices(locCartSk);
	
	for(tie(v1,v1_end) = vertices(locCartSk); v1!=v1_end; v1++){
		v2 = v1;v2++;
		for(; v2!=v2_end; v2++){
			if(edge(*v1,*v2,locCartSk).second){
				int eID_CSk = get(edgeID_CSk,edge(*v1,*v2,locCartSk).first);
				int comp = component[eID_CSk];
				if(S[*v1].size() == 1 || S[*v2].size() == 1){
					partial_marking_asCartesian(G, S[*v1], S[*v2], V_ID);
					//partial_marking_asnonCartesian(G, S[v_t], S[v_s], V_ID);
					partial_coloring(G, comp, S[*v1], S[*v2], V_ID, colors_of_edge, 
									 new_colors, ColorG, proper_color_conti);
				}
			}
			else if(S[*v1].size() == 1 || S[*v2].size() == 1){
				partial_marking_asnonCartesian(G, S[*v1], S[*v2], V_ID);
			}
		}
	}

	//print_graph(local_CartRelGraph);
	//myEdgeID edgeID_N = get(edge_index, thinN);
	//myEdgeID edgeID_CSk = get(edge_index, locCartSk);

/*		for(tie(ei,ei_end) = edges(locCartSk); ei!=ei_end; ei++){
		int v_t = target(*ei, thinN);
		int v_s = source(*ei, thinN);
		int eID_CSk = get(edgeID_CSk,edge(v_t,v_s,locCartSk).first);
		//cout << endl << v_s << "-" << v_t << "ID : " <<eID_CSk;
		int comp = component[eID_CSk];
		if(S[v_t].size() == 1 || S[v_s].size() == 1){
			partial_marking_asCartesian(G, S[v_t], S[v_s], V_ID);
			//partial_marking_asnonCartesian(G, S[v_t], S[v_s], V_ID);
			partial_coloring(G, comp, S[v_t], S[v_s], V_ID, colors_of_edge, new_colors, ColorG, proper_color_conti);
			
		}
		else{
			partial_marking_AsPossibleCartesian(G, S[v_t], S[v_s], V_ID);
		}
	}*/
//Tue 12 Jan 2010 03:46:14 PM CET 
	
	color_conti = true;
	if(num_old_colors == 0) color_conti = true;
	else
		for(unsigned int i = 0; i<new_colors.size();i++)
			if(!proper_color_conti[i]) color_conti = false;
		
	//cout << endl << "proper_color_conti " << color_conti;
	
}


void init_BFS_on_backbone_vertices(myGraph& G,vector< set<int> >& N,vector<int>& sigma){
	
	
	vIt vi, vi_end;
	set< set<int> > all_N;
	
	init_neighborhood(G, N);
	for(tie(vi,vi_end) = vertices(G); vi!=vi_end; vi++)
		all_N.insert(N[*vi]);
	
	//cout << endl << "max_set" << endl;
	vector< bool > is_backbone_vertex(num_vertices(G));
	for(tie(vi,vi_end) = vertices(G); vi!=vi_end; vi++){
		/**B(G) = set of vertices with strictly max neighborhood*/
		/**equal neighborhoods do not appear, thus suffices to look for maximality only*/
		is_backbone_vertex[*vi] = is_maximal_set(N[*vi], all_N);
	}
	
	//bool is_strictly_maximal_set(set<int>&, set< set<int> >& );
	//subgrap induziert durch B(G)	 und darauf BFS,
	vector<int> BFS_all;
	bfs(G, BFS_all);
	for(unsigned int i=0; i<BFS_all.size();++i) 
		if(is_backbone_vertex[BFS_all[i]])
			sigma.push_back(BFS_all[i]);
	


	/*Thu 11 Feb 2010 09:51:40 AM CET delete all elements from sigma
	that have same  closed neighborhood .. important for approx.stuff*/
	vector<int> sigma_copy;	//covering sequence
	sigma_copy=sigma;
	vector< bool > takeV(sigma.size());
	for(unsigned int i=0; i<sigma.size();++i)	
		takeV[i]=true;


	for(unsigned int i=0; i<sigma.size();++i){
		int v = sigma[i];
		for(unsigned int j=i+1; j<sigma.size();++j){
			int w = sigma[j];
			
			if(N[v] == N[w]){
				takeV[i]=false;
				takeV[j]=false;
			}
		}
	}

	sigma.clear();	
	for(unsigned int i=0; i<sigma_copy.size();++i){	
		if(takeV[i])	
			sigma.push_back(sigma_copy[i]);
	}
	/*Thu 11 Feb 2010 09:51:40 AM CET delete all elements from sigma
	that have same  closed neighborhood .. important for approx.stuff*/
	
}
	


bool init_thin_graph(myGraph& graph, myGraph& H, myGraph& GS, VecOfSets& S,
					vector<int>& c, int& gcd, bool& only_complete_factors, vector<int>& how_many){
	

	c.resize(num_vertices(graph));
	S=find_S(graph,c);/*cout<<"\nS classes:"<<endl;//showMulti(S);*/
	
	only_complete_factors = init_Gprime(graph,S,gcd,H,GS);
	
	if(only_complete_factors)
		return true;
	
	how_many.resize(S.size());
	
	GS=GoverS(graph,S,c,how_many, gcd);
	
	bool pumped_up = false;
	for(unsigned int i = 0; i<how_many.size(); i++) 
		if(how_many[i] > 1) pumped_up = true;
		
	return pumped_up;	

}


bool is_proper_subset(set<int>& Set1, set<int>& Set2){
    	/**test if S1 is subset of S2*/
		if(Set1.size() >= Set2.size()) return false;
		
		set<int>::iterator p;
		for(p = Set1.begin(); p != Set1.end(); ++p ){
			if(Set2.find(*p) == Set2.end()  ) return false;
		}
		return true;
}

bool is_subset(set<int>& Set1, set<int> &Set2){
    	/**test if S1 is subset of S2*/
		if(Set1.size() > Set2.size()) return false;
		
		set<int>::iterator p;
		for(p = Set1.begin(); p != Set1.end(); ++p ){
			if(Set2.find(*p) == Set2.end()  ) return false;
		}
		return true;
}

bool is_maximal_set_inMultiSet(set<int>& Subset, multiset< set<int> >& Superset){
	
	
	multiset < set<int> >::iterator p;
	set < int > T;
	for(p = Superset.begin(); p != Superset.end(); ++p){
		//if(is_proper_subset(Subset, *p)){  //????? warum geht das nicht????
		T.clear();
		T = *p;
		/*set<int>::iterator q;	 
		for( q = p->begin(); q != p->end(); ++q )
				T.insert(*q);*/
		
		//cout << endl << "test      :";showSet(Subset);
		//cout << endl << "is sub of :"; showSet(T);
		
		if(is_proper_subset(Subset, T)){ 
			return false;
		}
	}
	return true;
}

bool is_strictly_maximal_set_inMultiSet(set<int>& Subset, multiset< set<int> >& Superset){
	
	
	multiset< set <int> > tmp_J; tmp_J.clear();
	tmp_J = Superset;
	int old_size = tmp_J.size();
	tmp_J.erase(Subset);
	int new_size = tmp_J.size();
	
	if(new_size < (old_size - 1) )/**subset was more than ones included and thus is not strictly max */
		return false;
	
	tmp_J.clear();
	tmp_J = Superset;
	tmp_J.erase(Subset);
	
	multiset < set<int> >::iterator p;
	set < int > T;
	//for(p = Superset.begin(); p != Superset.end(); ++p){
	for(p = tmp_J.begin(); p != tmp_J.end(); ++p){
		//if(is_proper_subset(Subset, *p)){  //????? warum geht das nicht????
		set<int>::iterator q;	 
		T.clear();
		T=*p;
		//for( q = p->begin(); q != p->end(); ++q )
        //      T.insert(*q);
		
		//cout << endl << "test      :";showSet(Subset);
		//cout << endl << "is sub of :";showSet(T);
		
		if(is_subset(Subset, T)){ 
			return false;
		}
	}
	return true;
}

bool is_maximal_set(set<int>& Subset, set< set<int> >& Superset){
	
	
	set < set<int> >::iterator p;
	set < int > T;
	for(p = Superset.begin(); p != Superset.end(); ++p){
		//if(is_proper_subset(Subset, *p)){  //????? warum geht das nicht????
		T.clear();
		T = *p;
		/*set<int>::iterator q;	 
		for( q = p->begin(); q != p->end(); ++q )
				T.insert(*q);*/
		
		//cout << endl << "test      :";showSet(Subset);
		//cout << endl << "is sub of :"; showSet(T);
		
		if(is_proper_subset(Subset, T)){ 
			return false;
		}
	}
	return true;
}

bool is_strictly_maximal_set(set<int>& Subset, set< set<int> >& Superset){
	
	
	set < set<int> >::iterator p;
	set < int > T;
	for(p = Superset.begin(); p != Superset.end(); ++p){
		//if(is_proper_subset(Subset, *p)){  //????? warum geht das nicht????
		set<int>::iterator q;	 
		T.clear();
		T=*p;
		//for( q = p->begin(); q != p->end(); ++q )
        //      T.insert(*q);
		
		//cout << endl << "test      :";showSet(Subset);
		//cout << endl << "is sub of :";showSet(T);
		
		if(is_subset(Subset, T)){ 
			return false;
		}
	}
	return true;
}

void clear_graph(myGraph& G){

	vIt vi, vi_end, next;	
	tie(vi, vi_end) = vertices(G);
	for (next = vi; vi != vi_end; vi = next) {
		++next;
		clear_vertex(*vi, G);
	}
	
	tie(vi, vi_end) = vertices(G);
	for (next = vi; vi != vi_end; vi = next) {
		++next;
		remove_vertex(*vi, G);
	}
}

/*Thu 21 Feb 2013 04:02:39 PM CET 
void clear_sub_graph(sGraph& G){
	
	vIt vi, vi_end, next;	
	tie(vi, vi_end) = vertices(G);
	for (next = vi; vi != vi_end; vi = next) {
		++next;
		clear_vertex(*vi, G);
	}
	
	tie(vi, vi_end) = vertices(G);
	for (next = vi; vi != vi_end; vi = next) {
		++next;
		remove_vertex(*vi, G);
	}
}
*/

void mark_edges_noncartesian(myGraph &H_prime){
	
	property_map < myGraph, edge_weight_t >::type w = get(edge_weight, H_prime);
    int weights[1] = {0};
    int *wp = weights;
    graph_traits < myGraph >::edge_iterator e, e_end;
    for (boost::tie(e, e_end) = edges(H_prime	); e != e_end; ++e)
		w[*e] = *wp;
	
}

void mark_edges_unchecked(myGraph &H_prime){
	
	property_map < myGraph, edge_weight_t >::type w = get(edge_weight, H_prime);
    int weights[1] = {2};
    int *wp = weights;
    graph_traits < myGraph >::edge_iterator e, e_end;
    for (boost::tie(e, e_end) = edges(H_prime	); e != e_end; ++e)
		w[*e] = *wp;
	
}

bool init_Gprime(myGraph &graph,VecOfSets &S,int& gcd,myGraph &H, myGraph &GS){
	
	vector<int> complete_factors;
    H=Gprim(graph,S,gcd);
	
	//cout<<"\n\n>>> G without complete factors: <<<"<<endl;
	//print_graph(H);
	
	if(num_vertices(H)==1){
		//cout << "\n\n>>> only complete factors: <<< \n\n";
		//if(gcd>1) integer_pfd(gcd, complete_factors);
		//for(unsigned int i=0; i<complete_factors.size(); ++i)
		//	cout << "FactorNr " << i << ":  K_" << complete_factors[i] << "\n\n";
		return true;
	}
	
	return false;

}


void init_induced_neighborhood(myGraph& G,	VecOfSets& N, myGraph& induced_N, int vertex, vector<int>& V_ID){
	
	sGraph tmp;
	sGraph induced_N_tmp;
	copy_graph(G, tmp);
	induced_N_tmp = tmp.create_subgraph();
	//clear_graph(tmp);
	set<int>::iterator set_it;
	vIt vi, vi_end;
	V_ID.clear();
	
	for (set_it=N[vertex].begin();set_it!=N[vertex].end();++set_it){
	//	cout << "\nadd " << *set_it;
		add_vertex(*set_it,induced_N_tmp);
		V_ID.push_back(*set_it);
	}
	
	copy_graph(induced_N_tmp, induced_N);
}

void init_induced_H(myGraph& G,	VecOfSets& N, myGraph& induced_H, set<int>& V_H, vector<int>& V_ID){
	
	sGraph tmp;
	sGraph induced_N_tmp;
	copy_graph(G, tmp);
	induced_N_tmp = tmp.create_subgraph();
	//clear_graph(tmp);
	set<int>::iterator set_it;
	vIt vi, vi_end;
	V_ID.clear();
	
	for (set_it=V_H.begin();set_it!=V_H.end();++set_it){
	//	cout << "\nadd " << *set_it;
		add_vertex(*set_it,induced_N_tmp);
		V_ID.push_back(*set_it);
	}
	
	copy_graph(induced_N_tmp, induced_H);
}




void init_neighborhood(myGraph& G, vector< set<int> > &N){
	
	vIt vi, vi_end;
    aIt ai, ai_end;
	
	N.clear();
	N.resize(num_vertices(G));
	
	for (tie(vi, vi_end) = vertices(G); vi != vi_end; ++vi){
		/*closed neighbourhood*/
		N[*vi].insert(*vi); 
		for(tie(ai,ai_end) = adjacent_vertices(*vi,G); ai!=ai_end; ai++){
			N[*vi].insert(*ai);	
		}
	}
	
}

					  /**CartSk,CartRelGraph, .. */
void init_coordinates(myGraph& graph, myGraph& graph2, 
					  matrix&  coord_vert,
					  bool print_it){
	
	vector<int> component(num_vertices(graph2)); 
	int totalConnectComp=connected_components(graph2, &component[0]);
	
	coord_vert.clear();
	coord_vert.resize(num_vertices(graph));
	for(unsigned int i=0; i<num_vertices(graph); i++){
		coord_vert[i].resize(totalConnectComp);
	}
	
	eIt ei, ei_end;
	vIt vi, vi_end;
	myEdgeID edge_id = get(edge_index, graph);
		
	/* *******************BFS *********************************** */
	
	vIt s=vertices(graph).first;
	
	/*bool visited[num_vertices(graph)];
	vector<int> vert_on_level[num_vertices(graph)];
	queue<unsigned int> Q[num_vertices(graph)+1];
	queue<unsigned int> Q2[num_vertices(graph)+1];
	bool visited_edge[num_vertices(graph)][num_vertices(graph)];*/
	
	bool *visited = new bool[num_vertices(graph)];
	vector<int> *vert_on_level = new vector<int>[num_vertices(graph)];
	queue<int> *Q = new queue<int>[num_vertices(graph)+1];
	queue<int> *Q2 = new queue<int>[num_vertices(graph)+1];
	bool **visited_edge = new bool*[num_vertices(graph)];
	for (size_t i=0; i<num_vertices(graph); i++)
	  visited_edge[i] = new bool [num_vertices(graph)];
	
	vector<bool> is_labeled;
	is_labeled.resize(num_vertices(graph));
	
	int level = 0;
	for(tie(vi, vi_end)=vertices(graph); vi!=vi_end; ++vi)
		is_labeled[*vi]=false;	
	
	for(int i=0; i<totalConnectComp; ++i) 
		coord_vert[*s][i] = 0;
	
	is_labeled[*s]=true;
	
	/*Q2[0].push(*s);
	visited[*s]=true;
	vert_on_level[level].push_back(*s);*/
	

	/**BFS on unit-layer*/
	for(int i=0; i<totalConnectComp; ++i){
		level = 0;
		Q2[0].push(*s);
		visited[*s]=true;
		vert_on_level[level].push_back(*s);
	
		for(tie(ei, ei_end)=edges(graph); ei!=ei_end; ++ei){
				visited_edge[source(*ei,graph)][target(*ei,graph)]=false;
				visited_edge[target(*ei,graph)][source(*ei,graph)]=false;
		}
		
		for(tie(vi, vi_end)=vertices(graph); vi!=vi_end; ++vi)
			visited[*vi]=false;	
		
		int count=1;		
		while(!Q2[level].empty()){
		
			unsigned int u = Q2[level].front();
			//cout << endl << "level = " << level << " v= " << u;
			visited[u]=true;
			Q2[level].pop();
		
			aIt ai,ai_end;
		
			for(tie(ai, ai_end)=adjacent_vertices(u,graph);ai!=ai_end; ++ai){
				int ID=get(edge_id, edge(u,*ai, graph).first);
				
				if(!visited_edge[u][*ai] && !visited_edge[*ai][u] && component[ID]==i ){
					visited_edge[u][*ai]=true;
					visited_edge[*ai][u]=true;
					
					if(!visited[*ai]){
						
						vert_on_level[level+1].push_back(*ai);
					
 						for(int j=0; j<totalConnectComp;++j)
							coord_vert[*ai][j] = 0;
						
						coord_vert[*ai][i] = count;
						count++;
						Q2[level+1].push(*ai);
						visited[*ai]=true;
						is_labeled[*ai]=true;
					
					}	
				}
			}

			if(Q2[level].empty())
				level=level+1;
		}

	
		for(int l=0; l<level; ++l)
			vert_on_level[l].clear();	
	}

	level=0;
	
	for(tie(ei, ei_end)=edges(graph); ei!=ei_end; ++ei){
		visited_edge[source(*ei,graph)][target(*ei,graph)]=false;
		visited_edge[target(*ei,graph)][source(*ei,graph)]=false;
	}
	
	for(tie(vi, vi_end)=vertices(graph); vi!=vi_end; ++vi){
		visited[*vi]=false;	
//		for(int i=0; i<totalConnectComp; ++i) 
//			coord_vert[*vi][i]=0; 
	}
	
	for(int i=0; i<totalConnectComp; ++i) 
		coord_vert[*s][i] = 0;


	/**BFS on graph*/	
	Q[0].push(*s);
	visited[*s]=true;
	vert_on_level[level].push_back(*s);
		
	while(!Q[level].empty()){
		
		unsigned int u = Q[level].front();
		
		//cout << endl << "level = " << level << " v= " << u;
		
		visited[u]=true;
		Q[level].pop();
		
		
		aIt ai,ai_end;
	
		for(tie(ai, ai_end)=adjacent_vertices(u,graph);ai!=ai_end; ++ai){
			if(!visited_edge[u][*ai] && !visited_edge[*ai][u]){
				visited_edge[u][*ai]=true;
				visited_edge[*ai][u]=true;
				int ID=get(edge_id, edge(u,*ai, graph).first);
				
				//cerr << "\nlevel " << level << "("<< u << "," << *ai << ")  edgeID: "
				//	 << ID   << " in factor: "  << component[ID];

				if(!visited[*ai]){
					//cerr << "\n\n\n test " << *ai;// << " fiber " << component[ID];
					
					vert_on_level[level+1].push_back(*ai);
					Q[level+1].push(*ai);
					visited[*ai]=true;
					
					if(!is_labeled[*ai]){
						
						//cerr << endl << "label" << *ai;		
						aIt ai2,ai2_end;
						int second_u_vertex; //, tmp_2nd_u;
						//bool breaked = false;
						for(tie(ai2, ai2_end)=adjacent_vertices(*ai,graph);ai2!=ai2_end; ++ai2){
							int ID2=get(edge_id, edge(*ai,*ai2, graph).first);
							if(*ai2 != u && is_labeled[*ai2] && component[ID] != component[ID2]){ 
								second_u_vertex = *ai2;
								break;
							}
						}
						
						
						//cerr << "continue label" << second_u_vertex;
						for(int j=0; j<totalConnectComp;++j){
				
											
							int max;
							if(coord_vert[u][j] <= coord_vert[second_u_vertex][j]) 
								max = coord_vert[second_u_vertex][j];
							else max = coord_vert[u][j];
				
							coord_vert[*ai][j] = max;
						
						}
						is_labeled[*ai] = true;
					 }
				}	
			}
		 }
	  
		
		if(Q[level].empty())
			level=level+1;
	}	
	
	
	if(print_it)
		for(int l=0; l<level; ++l){
		
			cout << "\n\nlevel " << l << " -> ";
			for(unsigned int i=0; i<vert_on_level[l].size();++i){
				int V=vert_on_level[l][i];
				if(V<10) cout << "\ncoord of " << V << "  = ";
				else cout << "\ncoord of " << V << " = ";
				for(int j=0; j<totalConnectComp;++j){
					cout << coord_vert[V][j] << " ";
				}
			}
		}
		
	
	/* cleanup memory */
	delete [] visited;
	delete [] vert_on_level;
	delete [] Q;
	delete [] Q2;

	for (size_t i=0; i<num_vertices(graph); i++)
	  delete [] visited_edge[i];

	delete [] visited_edge;
	
	
	
}	

void init_components_wrt_RelGraph(myGraph& G, myGraph& CartSk, myGraph& CartRelGraph, 
								  vector<int>& component, vector<myGraph>& C, int numComp){
								  
	C.clear();
	C.resize(numComp);
	myEdgeID edge_id=get(edge_index,CartSk);
	myEdgeW w = get(edge_weight, G);
	
		
	/*print_graph(G);
	cout << endl;

	print_graph(CartSk);
	cout << endl;*/
	
	//print_graph(CartSk);
	for(int i = 0; i<numComp; i++){
		eIt ei, ei_end;
		for(tie(ei,ei_end) = edges(G); ei!=ei_end; ei++){
			unsigned int v_target=target(*ei,G);
			unsigned int v_source=source(*ei,G);
			



			if(w[edge(v_target,v_source,G).first] == 1  /**check if Cartesian*/
				/*Fri 15 Jan 2010 12:55:05 PM CET */
				&& num_vertices(CartSk) > v_target
				&& num_vertices(CartSk) > v_source
				&& edge(v_target,v_source,CartSk).second){
				/*Fri 15 Jan 2010 12:55:05 PM CET */
			
				/*if(int(num_vertices(CartSk)) <= v_target)
					for(int k = int(num_vertices(CartSk)); k <= v_target;k++)
							add_vertex(CartSk);
					
					
				if (int(num_vertices(CartSk)) <= v_source)
					for(int k = int(num_vertices(CartSk)); k <= v_source;k++)
							add_vertex(CartSk);
					
				
				int eid = num_edges(CartSk);
				if(!edge(v_target,v_source,CartSk).second)
					add_edge(v_target,v_source,eid,CartSk);
				*/
			
				int id = get(edge_id, edge(v_target,v_source,CartSk).first);
				if(component[id] == i)
				{
					if(num_vertices(C[i]) <= v_target)
						for(unsigned int j = num_vertices(C[i]); j <= v_target;j++)
							add_vertex(C[i]);
		
					if (num_vertices(C[i]) <= v_source)
						for(unsigned int j = num_vertices(C[i]); j <= v_source;j++)
							add_vertex(C[i]);

					add_edge(v_target,v_source,C[i]);	
				}
			}
			
		}
	}
	

}


void init_newCartSk(myGraph& G,myGraph& CartSk,myGraph& CartSk_edgeID,myGraph& CartRelGraph){
	
	/**IDEA:  run over components c_1, .. , c_n of RelGraph .. add_egdes in graph C_1,,C_n
			take components ci_1,.. , ci_ni   of each C_i, compute induced subgraph in GS
			than add CartRelGraph vertices connect and so on. .. */

	vector<int> component(num_vertices(CartRelGraph));
	vector<myGraph> C;
	int numComp = connected_components(CartRelGraph, &component[0]);

	
		
	init_components_wrt_RelGraph(G ,CartSk,CartRelGraph, component,C, numComp);
	
	
	//clear_graph(CartSk);
	//clear_graph(CartSk_edgeID);
	
	//myEdgeID edge_id=get(edge_index,CartSk);
	//myEdgeID edge_id_orig=get(edge_index,CartSk_edgeID);
	
	sGraph sub_G;
	//clear_sub_graph(sub_G);Thu 21 Feb 2013 04:02:39 PM CET 
	copy_graph(G, sub_G);
	//mark_edges_noncartesian(G);
	
			
	clear_graph(CartSk);clear_graph(CartSk_edgeID);clear_graph(CartRelGraph);
	//add_vertex(CartSk);add_vertex(CartSk_edgeID);
	//add_vertex(CartRelGraph);
	
	for(int i = 0; i<numComp; i++){
		//cout << "C_" << i << endl;
		//print_graph(C[i]);
		
		bool newFactor=true;
		vector<int> component_Ci; component_Ci.clear();
		component_Ci.resize(num_vertices(C[i]));
		int numComp_Ci = connected_components(C[i], &component_Ci[0]);
		//cout << "#Comp = " << numComp_Ci;
		
		
		sGraph *graphComp_Ci = new sGraph[numComp_Ci];
		for(int j=0; j<numComp_Ci; ++j){
			//clear_sub_graph(graphComp_Ci[j]);Thu 21 Feb 2013 04:02:39 PM CET 
			graphComp_Ci[j]=sub_G.create_subgraph();
		}
		
		
		vIt vi, vi_end;
		for(tie(vi, vi_end)=vertices(C[i]); vi!=vi_end; ++vi){
			int j = component_Ci[*vi];
		//	cout << endl << "v = " << *vi << " in component " << j;
			myVertexName_sub vID = get(vertex_name, graphComp_Ci[j]);
			vD  v = add_vertex(*vi,graphComp_Ci[j]);
			put(vID, v, *vi);		
		}
		
					
		for(int j=0; j<numComp_Ci; ++j){
			//cout << j << endl;print_graph(graphComp_Ci[j]);
			myVertexName_sub vID = get(vertex_name, graphComp_Ci[j]);
			eIt ei, ei_end;
			for(tie(ei, ei_end)=edges(graphComp_Ci[j]); ei!=ei_end; ++ei){
				int sourceID = vID[source(*ei, graphComp_Ci[j])];
				int targetID = vID[target(*ei, graphComp_Ci[j])];
				
				if(int(num_vertices(CartSk)) <= sourceID)
					for(int k = int(num_vertices(CartSk)); k <= sourceID;k++){
							add_vertex(CartSk);add_vertex(CartSk_edgeID);
					}
					
				if (int(num_vertices(CartSk)) <= targetID)
					for(int k = int(num_vertices(CartSk)); k <= targetID;k++){
							add_vertex(CartSk);add_vertex(CartSk_edgeID);
					}
				
				if(!edge(sourceID,targetID,CartSk).second){
					
					property_map < myGraph, edge_weight_t >::type w = get(edge_weight, G);
					int weights[1] = {1};
					int *wp = weights;
					w[edge(sourceID,targetID,G).first] = *wp;
					
					int numE = num_edges(CartSk);
					//cout << endl << "edge ("<< sourceID << "," << targetID << ") getID " << numE;
					add_edge(sourceID,targetID,numE,CartSk);
					
					myEdgeID edge_id=get(edge_index,G);
					add_edge(sourceID,targetID,get(edge_id, edge(sourceID,targetID,G).first),CartSk_edgeID);
					
					add_vertex(CartRelGraph);
					if(!newFactor) 
						add_edge((numE-1),numE,CartRelGraph);
					
					newFactor=false;
				}
			}
		}
	}
}

void init_factor_corresponding_to_coordinates( matrix  coord_vert,
											   myGraph& GS, int numFactors, 
											   vector<myGraph>& F, bool print_it){
	/**init factors. corresponding to coordinates .. */
	
	vIt vi, vi_end;
	vector< bool > fix_v(num_vertices(GS));
	F.clear();
	F.resize(numFactors);
	
	//4:37 PM 6/4/2009
	/*for(unsigned int i=0; i<numFactors;++i)
		for(tie(vi,vi_end) = vertices(new_factors[i]); vi!=vi_end; vi++) 
			add_vertex(F[i]);*/
	//4:37 PM 6/4/2009
	
	for(tie(vi,vi_end) = vertices(GS); vi!=vi_end; vi++) fix_v[*vi] = true;
	
	for(int i=0; i<numFactors;++i){ //cerr << endl <<" factor " << i;
		for(tie(vi,vi_end) = vertices(GS); vi!=vi_end; vi++){
			for(int j=0; j<numFactors;++j){
				if(i!=j && coord_vert[*vi][j] != 0){
	//				cout << endl << *vi << " i= " << i << " j = " << j << " coord " << coord_vert[*vi][j] ; 
					fix_v[*vi] = false;
				}
			}
		}
		
		for(tie(vi,vi_end) = vertices(GS); vi!=vi_end; vi++){
			if(fix_v[*vi]){ //cerr << "\nfix" << *vi;
				aIt ai, ai_end;
				for(tie(ai,ai_end) = adjacent_vertices(*vi,GS); ai!=ai_end; ai++){
					if(int(num_vertices(F[i])) <= coord_vert[*vi][i])
						for(int j = num_vertices(F[i]); j <= coord_vert[*vi][i];j++)
							add_vertex(F[i]);
						
					if (int(num_vertices(F[i])) <= coord_vert[*ai][i])
						for(int j = num_vertices(F[i]); j <= coord_vert[*ai][i];j++)
							add_vertex(F[i]);
						
					
					if(fix_v[*ai] && !edge(coord_vert[*vi][i],coord_vert[*ai][i],F[i]).second 
						&& coord_vert[*vi][i] != coord_vert[*ai][i]){
						//	cerr << endl <<  " add " << *vi << "-" << *ai; 
						//	cerr << endl <<  " addF " << coord_vert[*vi][i]  << "-" << coord_vert[*ai][i]; 
							add_edge(coord_vert[*vi][i],coord_vert[*ai][i],F[i]);
					}
				}
				
			}
			fix_v[*vi] = true;
		}
	}
	
	if(print_it){
		cout << endl;
		for(int i=0; i<numFactors;++i){
			cout << endl;
			print_graph(F[i]);
		}
	}
}

void stop(){
	cout << endl <<"stop";
	cout << endl;
}


void init_ALLSubsets(int n, vector<int>& bin, vector<int>& elements, vector< set<int> >& Pow){

	int i=0;
	set<int> tmp;
	
	//cout << endl << "bin.size = " << bin.size();
	
	while(i<exp(n*log(2))){//no: of subset is 2^n where n is the no:of elements
		
		tmp.clear();
        for(int k=0;k<n;k++){
            if(bin[k]==1) 					
				tmp.insert(elements[k]);
				//cout << " " << elements[k];//outputs if bin[k]=1
        }
		Pow.push_back(tmp);
		
		int j=0;
        bin[j]=bin[j]+1;
		//bin.at(j)=bin.at(j)+1;
        while(bin[j]>1){ //generating binary sequence,000,001,010,011.....
		  //while(bin.at(j)>1){ //generating binary sequence,000,001,010,011.....
		   
		   
		 //  cout << endl << "j = " << j;
		   
           bin[j]=bin[j]-2;
		   //bin.at(j)=bin.at(j)-2;
           j=j+1;
		   if(j>=int(bin.size())) break;
		   //bin.at(j)=bin.at(j)+1;
           bin[j]=bin[j]+1;
		   //if(j>=n) break;
		   
        }

        i=i+1;
    }
}

void init_Sets_of_given_lenght(int l, vector< set<int> >& Pow, vector< set<int> >& Set_L ){

	for(unsigned int i=0;i<Pow.size();i++){
		if(int(Pow[i].size()) == l){
			Set_L.push_back(Pow[i]);
		}
	}

}


void init_cart_skeleton_graph_afterHammack(myGraph &G, myGraph &CartSk, myGraph &CartSk_edgeID){
	
	eIt ei, ei_end;
	property_map < myGraph, edge_weight_t >::type w = get(edge_weight, G);
	myEdgeID edge_id=get(edge_index,G);
	//int edgeID=0;
    
	int edgeID=num_edges(CartSk);
	
	for (tie(ei, ei_end) = edges(G); ei != ei_end; ++ei){
		  if(w[edge(source(*ei,G),target(*ei,G),G).first]==1){
			if(!edge(source(*ei,G),target(*ei,G),CartSk).second){
				add_edge(source(*ei, G),target(*ei, G),get(edge_id, *ei),CartSk_edgeID);
				add_edge(source(*ei, G),target(*ei, G),edgeID,CartSk);
				++edgeID;
			}
		 }
	}

}

void init_intersection(set<int>& N1, set<int>& N2,set<int>& N_inters){

	N_inters.clear();
	set_intersection(N1.begin(), N1.end(),
					 N2.begin(), N2.end(),
					inserter(N_inters, N_inters.begin()));

}

void init_union(set<int>& N1, set<int>& N2,set<int>& N_inters){

	N_inters.clear();
	set_union(N1.begin(), N1.end(),
					 N2.begin(), N2.end(),
					inserter(N_inters, N_inters.begin()));

}

void init_union_all(set<int>& N1, set<int>& N2,set<int>& N_inters){

//	N_inters.clear();
	set_union(N1.begin(), N1.end(),
					 N2.begin(), N2.end(),
					inserter(N_inters, N_inters.begin()));

}

void take_biggest_colored_component_as_factor(myGraph& G, myGraph& CartSk, 
											  myGraph& CartRelGraph, vector<myGraph>& FACTORS,
											  myGraph& ColoredCartSk){
	

	
	vector<int> comp(num_vertices(CartRelGraph));
	int num_factors = connected_components(CartRelGraph, &comp[0]);
	/*vector<sGraph> subgraph_factors(num_factors);*/
	
	FACTORS.resize(num_factors);
	//cout << endl << "numF = " << num_factors;      
	
	vector<sGraph> factors(num_factors);
						 
	eIt ei, ei_end;
	vIt vi, vi_end;
										 
	myEdgeW weightCCS = get(edge_weight, ColoredCartSk);
	myEdgeID edgeID_CSk = get(edge_index, CartSk);
	vector<myGraph> colored_factor(num_factors);
	for(tie(ei, ei_end)=edges(CartSk); ei!=ei_end; ei++){
		//if(get(weightG,*ei) == 1){
			int layer = comp[get(edgeID_CSk, *ei)];
			
			//cout << endl << *ei << " ID in CSk =" << get(edgeID_CSk, *ei) 
			//	 <<" in layer of factor " << comp[get(edgeID_CSk, *ei)] << endl;
			if (num_vertices(colored_factor[layer]) <= target(*ei,CartSk))
			    for(unsigned int j = num_vertices(colored_factor[layer]); j <= target(*ei,CartSk);j++)
							add_vertex(colored_factor[layer]);
			
			if (num_vertices(colored_factor[layer]) <= source(*ei,CartSk))
			    for(unsigned int j = num_vertices(colored_factor[layer]); j <= source(*ei,CartSk);j++)
							add_vertex(colored_factor[layer]);
					
			add_edge(source(*ei,CartSk),target(*ei,CartSk), colored_factor[layer]);
	
			add_edge(source(*ei,CartSk),target(*ei,CartSk),get(edgeID_CSk,*ei), ColoredCartSk);
			

		//}
	}
	
	
	//myEdgeW weightCCS = get(edge_weight, ColoredCartSk);
	for(tie(ei,ei_end) = edges(ColoredCartSk); ei!=ei_end; ++ei){
			int layer = comp[get(edgeID_CSk, *ei)];
			put(weightCCS, *ei, layer); 
			//cout << endl << *ei << " - " << get(weightCCS,*ei);			
	}
	
	sGraph tmp_graph;
	copy_graph(G,tmp_graph);
	
	for(int i=0; i<num_factors; ++i){
		
		factors[i]=tmp_graph.create_subgraph();	
		//cout << "\n\n G" << i << endl;
		vector<int> tmp(num_vertices(colored_factor[i]));
		tmp.clear();
		int tmp_num = connected_components(colored_factor[i], &tmp[0]) ;
		
		vector<int> count_maxV;
		vector<int> count_maxE;
		count_maxV.resize(tmp_num);
		count_maxE.resize(tmp_num);
		for(int j=0; j<tmp_num;++j){ 
				count_maxV[j] = 0;
				count_maxE[j] = 0;
		}
		
		/*count max number of vertices in each j-th copy of each i-layer*/
		for(tie(vi, vi_end)=vertices(colored_factor[i]); vi!=vi_end; vi++){
			for(int j=0; j<tmp_num;++j){
				if(tmp[*vi] == j) ++count_maxV[j];
			}
		}
		
		
		/*count max number of edges in each j-th copy of each i-layer*/
		//cout << endl << "check copies of "<<i<<"-layer" << endl;
		//print_graph(colored_factor[i]);
		//cout << endl;
		for(tie(ei, ei_end)=edges(colored_factor[i]); ei!=ei_end; ei++){
			vD v = target(*ei,colored_factor[i]);
			//cout << "check : " << *ei << "target:"<< v;
			for(int j=0; j<tmp_num;++j){
				if(tmp[v] == j){
					 ++count_maxE[j];
				//	cout << " comp: " << j <<endl;
				}
			}
		}
	
		
		
		vector<int> checkPossible_maxV_Layer;
		int maxV = 0;
		for(int j=0; j<tmp_num;++j){
			if(count_maxV[j]>=maxV){
				maxV = count_maxV[j];
			}
		}	

		//cout << endl <<"maxV	" <<endl;
		for(int j=0; j<tmp_num;++j){
			if(count_maxV[j]==maxV){
				//cout << endl << " " << j << "- maxV" << maxV;
				checkPossible_maxV_Layer.push_back(j);
			}
		}		

		
		int maxE = 0;
		int take_this = 0;
		for(unsigned int j=0; j<checkPossible_maxV_Layer.size();++j){
			if(count_maxE[checkPossible_maxV_Layer[j]]>=maxE){
				maxE = count_maxE[checkPossible_maxV_Layer[j]];
				//take_this = j;
			}
		}
	
		
		for(unsigned int j=0; j<checkPossible_maxV_Layer.size();++j){
		//for(int j=0; j<tmp_num;++j){
			//cout << endl <<  " " << j << "-" << maxE;
			if(count_maxE[checkPossible_maxV_Layer[j]]==maxE){
				take_this = checkPossible_maxV_Layer[j];
			}
		}		


		//int case_of_max_element;Thu 14 Jan 2010 10:04:33 AM CET */
		/*int take_this = 0;
		int max = 0;
		for(int j=0; j<tmp_num;++j){
		//	cout << "\nj= " << j << "num_el = " << count_maxV[j];
			if(count_maxV[j]>=max){
				max = count_maxV[j];
				take_this = j;
			}
		}Thu 14 Jan 2010 10:04:33 AM CET */
		
		/**search for connected components that are more than single vertex, exists in nonNICE graphs (u.dat)*/
		/*int take_this = 0;
		for(tie(vi, vi_end)=vertices(colored_factor[i]); vi!=vi_end; vi++){
				if(out_degree(*vi, colored_factor[i]) > 0){
					take_this = tmp[*vi];
					break;
			} 
		}*/
		//cout << "\nnum: " << num << endl;
		//print_graph(colored_factor[i]);

		for(tie(vi, vi_end)=vertices(colored_factor[i]); vi!=vi_end; vi++){
			if(tmp[*vi]==take_this)
					add_vertex(*vi, factors[i]); 
		}
		copy_graph(factors[i],FACTORS[i]);	
	}
	
	//cout << "\n possible factors are : \n";
	/*for(int i=0; i<num_factors; ++i){
		cout << "\nG" << i << endl; print_graph(FACTORS[i]);
	}*/	
	

}

