#include "definitions.h"
#include "integer_pfd.h"
#include "init_stuff.h"
#include "print_stuff.h"

#include <boost/graph/graph_utility.hpp>
#include <boost/graph/connected_components.hpp>

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
//#include <sstream>

using namespace std;
using namespace boost;

void print_approxFactors(vector<myGraph> new_factors){

	cout << endl << endl << ">>> ***APPROXIMATE FACTORS of QuotientGraph G*** <<<" << endl;
					cout << ">>>    Extracted Factors of Cartesian Skeleton   <<<" << endl;
					cout << ">>>    (taking largest component of each color   <<<" << endl;
					cout << ">>>     without using Nstar-neighborhoods)       <<<" << endl;

	for(unsigned int i=0; i<new_factors.size(); ++i){
			cout << endl <<"factor " << i << ":" << endl;
			print_graph(new_factors[i]); cout << endl;
	}
}

void print_Sigma(vector<int> sigma){

	cout << "\ncovering-ordering (BFS) of backbone_vertices: " << endl;
	for(unsigned int i=0; i<sigma.size();++i){
		cout << " " << sigma[i];
		//cout << " N = "; showSet(N[sigma[i]]);
		if(i%10==0 && i!=0) cout << endl;
	}

}

int print_OnlyCompleteFactors_and_QuotientGraph(bool only_complete_factors,int gcd, 
												 myGraph& G,myGraph& graph){
	if(only_complete_factors){
		print_only_compl_factors(gcd);
		return 0;
	}

	if(num_vertices(G)!=num_vertices(graph)){
		cout << endl << "\nG = H/S: " << endl;
		print_graph(G);
	}
	return 1;
}

void print_only_compl_factors(int gcd){
		vector<myGraph> empty; empty.clear();
		printPFD(gcd,empty);
		cout << "     ONLY complete Factors"  << endl <<endl;

}

void print_N(myGraph& induced_N, int current_v ){
	cout << endl << endl << "Compute PFD of <N["<< current_v <<"]> : " << endl << endl;
	print_graph(induced_N);
	cout << endl;
}

void print_local_Factors(myGraph& local_CartSk, myGraph& local_CartRelGraph){

		vector<int> local_component(num_vertices(local_CartRelGraph));
		int numero = connected_components(local_CartRelGraph, &local_component[0]);
		vector<myGraph> tmp_new_factors;
		tmp_new_factors.clear();
		vector< vector<int> > tmp_coord;
		tmp_coord.clear();
		init_coordinates(local_CartSk, local_CartRelGraph, tmp_coord, false);
		cout << endl <<">>> Factors of Cartesian Skeleton <<<" << endl;
		init_factor_corresponding_to_coordinates(tmp_coord, local_CartSk, numero, tmp_new_factors, true);
		
}

void print_end_local(bool proper_color_conti, int count_Thin_N, int count_nonThin_N, int count_prime_N, 
					int count_Nstar, int count_edgeN){
	
	cout << "\n\n" <<  "--------------------------------------" << endl;
	//cout           <<   ">>> *   End of LocalApproach  * <<<" << endl;
	cout           <<   ">>> *End of Neighborhood Covering* <<<" << endl;
	
	
	cout << endl << "INTERMEDIATE INFORMATIONS:" << endl;
	if(proper_color_conti) cout << endl << "  proper_color_continuation     : YES";
	else                   cout << endl << "  proper_color_continuation     : NO";
	cout << endl << "  non-prime 1-Neighborhoods  ";
	cout << endl << "\t number of thin N       : " << count_Thin_N;
	cout << endl << "\t number of non-thin N   : " << count_nonThin_N;
    cout << endl << "  prime 1-Neighborhoods \t: " << count_prime_N;
	cout << endl << "  other Neighborhoods  	  ";
    cout << endl << "\t number of used edgeN   : " << count_edgeN;
	cout << endl << "\t number of used N*      : " << count_Nstar;
	

	
	cout  << endl << "--------------------------------------" << endl;
	cout << endl;
}


void printPFD(int gcd, vector<myGraph>& new_factors){
		//cout << "\n\n" <<  ">>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<" << endl;
		cout << "\n\n" <<  "------------------------------------" << endl;
		cout           <<   ">>> *End of PrimeFactorization* <<<" << endl;
		cout  <<  "------------------------------------" << endl;
		//cout <<   ">>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<" << endl <<endl;
		
		vector<int> complete_factors;
		if(gcd>1){ 	
			integer_pfd(gcd, complete_factors);
			print_only_completeFactors(complete_factors);
		}
		
		print_factor(gcd, new_factors);

}

void print_only_completeFactors(vector<int>& complete_factors){
	cout << endl << ">>> complete Factors: <<<" << endl;
	for(unsigned int i = 0; i < complete_factors.size();i++){
		cout << endl << "K_" << complete_factors[i];// << " " << endl;
	}
}

void print_factor(int gcd,vector<myGraph>& new_factors){
	if(new_factors.size()>=1 && gcd>1){
		cout << endl << ">>> non-complete Factors are: <<<" << endl;
		for(unsigned int i=0; i<new_factors.size();i++){
			cout << endl << "** G_" << i << " **" << endl;
			print_graph(new_factors[i]);
		}
	}
	else  if(new_factors.size()>1){
		cout << endl << ">>> non-complete Factors are: <<<" << endl;
		for(unsigned int i=0; i<new_factors.size();i++){
			cout << endl << "** G_" << i << " **" << endl;
			print_graph(new_factors[i]);
		}
	}
	else if (new_factors.size() == 1 && gcd == 1) 
		cout << endl << "Graph is Prime";
}


void show_local_S_class(myGraph& locCartSk, VecOfSets& S, vector<int>& V_ID){
	
	vIt vi, vi_end;
	for(tie(vi,vi_end) = vertices(locCartSk); vi!=vi_end; vi++){
		cout << "\n S[" << *vi << "] = ";
		set<int>::iterator its;
		for (its=(S[*vi]).begin();its!=(S[*vi]).end();++its)
			cout<<V_ID[*its]<<" ";
		cout << "\n        ";
		for (its=(S[*vi]).begin();its!=(S[*vi]).end();++its)
			cout<<*its<<" ";
	}	
}

		
void showSet(set<int>& s)
{
   set<int>::iterator its;
   
   for (its=s.begin();its!=s.end();++its)
    cout<<*its<<" ";
   //cout<<endl;
}

//***************************************************************************

void showMulti(VecOfSets& m)
{
   vosIt itm;
   cout << " { ";
   for (itm=m.begin();itm!=m.end();++itm){
	cout << " { ";
	showSet(*itm);
	cout << " } ";
   }
   cout << " } ";
   cout<<endl; 
    
}

 //void print_allsubsets( set< set<int> >& sos ){
 void showMultiSet(set< set<int> >& sos){
     
	 set< set<int> >::iterator p;
	 
     for( p = sos.begin(); p != sos.end(); ++p ){
         cout << " { ";
		 set<int>::iterator q;	 
         for( q = p->begin(); q != p->end(); ++q )
              cout << *q << ", ";
         cout << " } ";
     }
	 
 }

