#include"definitions.h"
#include"test_and_combine.h"
#include"init_stuff.h"
#include"print_stuff.h"

#include <vector>
#include <boost/config.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graph_utility.hpp>
#include <boost/graph/connected_components.hpp>
//#include <boost/graph/subgraph.hpp>
#include <boost/graph/copy.hpp>
 
using namespace std;
using namespace boost;


bool test_directEdges(myGraph& G, myGraph& CartSk, myGraph& CartRelGraph){
	/**check if crossedges exists*/
	
	
	vector<int> component(num_vertices(CartRelGraph));
	connected_components(CartRelGraph, &component[0]);
	
	myEdgeID edge_id=get(edge_index,CartSk);
	vector< vector<bool> > compared(num_edges(CartSk));
	
	eIt ei, ei_end;
	vIt vi, vi_end;
	for(tie(ei,ei_end) = edges(CartSk); ei!=ei_end; ei++){
		int id1 = get(edge_id, *ei);
		compared[id1].resize(num_edges(CartSk));
		eIt ei2, ei2_end;
		for(tie(ei2,ei2_end) = edges(CartSk); ei2!=ei2_end; ei2++){;
			int id2 = get(edge_id, *ei2);
			compared[id1][id2]=false;
		}
	}
	
	
	bool combined = false;
	
	for(tie(ei,ei_end) = edges(CartSk); ei!=ei_end; ei++){
		unsigned int v_target=target(*ei,G);
		unsigned int v_source=source(*ei,G);
		int id1 = get(edge_id, *ei);
		aIt ai,ai_end;
		
		for(tie(ai,ai_end) = adjacent_vertices(v_target,CartSk); ai!=ai_end; ai++){
			int id2 = get(edge_id, edge(v_target,*ai,CartSk).first);
			if(*ai!=v_source && !compared[id1][id2]){
				compared[id1][id2]=true;
				compared[id2][id1]=true;
				
				if(component[id1]!=component[id2] && !edge(v_source,*ai,G).second){
					combined = true;
					/*cout << endl << "combine colors" << component[id1] << "and" << component[id2] 
					     << "addEgde in RelGraph" << id1 << "-" << id2;*/
					//add_edge(id1, id2, CartRelGraph);
				}
			}
		}
		
		for(tie(ai,ai_end) = adjacent_vertices(v_source,CartSk); ai!=ai_end; ai++){
			int id2 = get(edge_id, edge(v_source,*ai,CartSk).first);
			if(*ai!=v_target && !compared[id1][id2]){
				compared[id1][id2]=true;
				compared[id2][id1]=true;
				
				if(component[id1]!=component[id2] && !edge(v_target,*ai,G).second){
					combined = true;
					/*cout << endl << "combine colors" << component[id1] << "and" << component[id2]
						 << "addEgde in RelGraph" << id1 << "-" << id2;*/
					//add_edge(id1, id2, CartRelGraph);
				}
			}
		}
		
	}
	
	return (!combined);
}


	
bool test_isomorphism(myGraph& G,vector< vector<int> >& coord_vert, vector<int>& component, int& numFactors,
							 vector<myGraph>& factors, myGraph& CartSk, myGraph& CartRelGraph){
	
	//myEdgeID edge_id=get(edge_index,CartSk);
	myEdgeID edge_id=get(edge_index,G);
	myEdgeID edge_id_Cart=get(edge_index,CartSk);
	vector<bool> checked_edge(num_edges(G));
	
	
	eIt ei, ei_end;
	vIt vi, vi_end;
	for(tie(ei,ei_end) = edges(G); ei!=ei_end; ei++){
		int id = get(edge_id, *ei);
		checked_edge[id] = false;
	}
	
	
	
	
	
	property_map < myGraph, edge_weight_t >::type w = get(edge_weight, G);
	/**check for all Cartesian edges in G of G_i-layer, if edge exists in factor G_i*/
	for(tie(ei,ei_end) = edges(G); ei!=ei_end; ei++){
	//for(tie(ei,ei_end) = edges(CartSk); ei!=ei_end; ei++){
		int cart_weight = get(w,*ei);
		if(cart_weight == 1){
			
			//cerr << endl <<  "check vertices " << v_target << " - " << v_source;
			unsigned int v_target=target(*ei,G);
			unsigned int v_source=source(*ei,G);
			int id = get(edge_id_Cart, edge(v_target,v_source,CartSk).first);
			//cerr <<  " edgeID of CartSk " << id ;
			
			int v1 = coord_vert[v_target][component[id]];
			int v2 = coord_vert[v_source][component[id]];
			
			//cerr <<  endl << " factor vertices : " << v1 << " " << v2 ;		
			if(!edge(v1,v2,factors[component[id]]).second){
			//		cout << "ERROR";
					return false;
			}
			else checked_edge[id] = true;
		}
	}
	

	
	/**check now if all edges of factor G_i are also appearing in all G_ifiber*/
	for(int i = 0; i<numFactors; i++){
		for(tie(ei,ei_end) = edges(factors[i]); ei!=ei_end; ei++){
			//myEdgeID edge_id_F=get(edge_index,factors[i]);
			//int id = get(edge_id_F, *ei);
			int v_target=int(target(*ei,factors[i]));
			int v_source=int(source(*ei,factors[i]));
			eIt ei1, ei1_end;
			int how_often = 0;
			for(tie(ei1,ei1_end) = edges(G); ei1!=ei1_end; ei1++){
				int cart_weight = get(w,*ei1);
				if(cart_weight == 1){
					unsigned int v1_target=target(*ei1,G);
					unsigned int v1_source=source(*ei1,G);
					if( (coord_vert[v1_target][i] == v_target && coord_vert[v1_source][i]==v_source)
						||(coord_vert[v1_target][i] == v_source && coord_vert[v1_source][i]==v_target)){
						//cout << endl << *ei ;
						++how_often;
					}
				 }
			}
			
			int num_vert_CoFactor = 1;
			for(int j = 0; j<numFactors; j++){
				if(j!=i)
					num_vert_CoFactor *= num_vertices(factors[j]);
			}
			//cout << endl << "num_VerticesCoF = " << num_vert_CoFactor << "?=" << how_often << endl;	
			if(how_often != num_vert_CoFactor) return false;
			
		}
	}
	
	bool test_dirEdges = test_directEdges(G, CartSk, CartRelGraph);
	return test_dirEdges;
}

bool init_and_test_isom_H1_H2(const myGraph& CartRel,/*const*/ myGraph& G,const set<int>& S, 
							  const set<int>& I_minus_S,const vector<int>& component,
							  const vector< vector<int> >& coord_vert,
							  const myGraph& CartSk,const myGraph& CartSkEdgeID){

	
	 myGraph tmp_CartRel;
	 clear_graph(tmp_CartRel);
	 copy_graph(CartRel,tmp_CartRel);

	/**first new RelGraph, by connecting components induces by set S and set I_minus_S*/
	eIt ei, ei_end;
	
	vIt vi,vi_end, vi2, vi2_end;
	set<int>::iterator its, S_begin;
	set<int> TT = S;
	//cout << "again "; showSet(TT);
	S_begin = S.begin();
	its=S.begin(); 
	++its;
	if(S_begin!=S.end()){
		//cout << endl << "1." << *S_begin;
		for(tie(vi,vi_end) = vertices(CartRel); vi!=vi_end; ++vi){
		//	cout << " " <<*vi << "in comp: "  << component[*vi];
			if(component[*vi] == *S_begin){
			//	cout << " check"; 
				//if(its == S.end()) cout << " nothing done";
				while(its!=S.end()){
					//cout << endl << "2." << *its;
					for(tie(vi2,vi2_end) = vertices(CartRel); vi2!=vi2_end; ++vi2)
							if(component[*vi2] == *its)
								if(!edge(*vi,*vi2,tmp_CartRel).second){
									add_edge(*vi,*vi2,tmp_CartRel);
								}
					++its;
				}
				
				break;			
			}
		}	
	}
	
	
	
	S_begin = I_minus_S.begin();
	its=I_minus_S.begin(); 
	++its;
	
	if(S_begin!=I_minus_S.end() ){
		//cout << endl << "1." << *S_begin;
		for(tie(vi,vi_end) = vertices(CartRel); vi!=vi_end; ++vi){
			//cout << endl <<"1.component_vert "<< *vi <<" = " << component[*vi];
			if(component[*vi] == *S_begin){
				while(its!=I_minus_S.end()){
					//cout << endl << "2." << *its;
					for(tie(vi2,vi2_end) = vertices(CartRel); vi2!=vi2_end; ++vi2){
						//cout << endl <<"2.component_vert" << *vi2 << " = " << component[*vi2];
						if(component[*vi2] == *its){
							if(!edge(*vi,*vi2,tmp_CartRel).second){
								add_edge(*vi,*vi2,tmp_CartRel);
							}
						}
					}
					++its;
				}		
			break;
			}
		}
	}	
	
	/**after new RelGraph, by connecting components induces by set S and set I_minus_S
	    compute new CartSk etc.pp.*/
	myGraph tmp_CartSk;
	myGraph tmp_CartSk_edgeID;
	clear_graph(tmp_CartSk);
	clear_graph(tmp_CartSk_edgeID);
	copy_graph(CartSk,tmp_CartSk);
	copy_graph(CartSkEdgeID,tmp_CartSk_edgeID);
	
	myGraph tmpG;
	clear_graph(tmpG);
	copy_graph(G,tmpG);
	/**also newRelGraph is new initialisert*/
	init_newCartSk(tmpG,tmp_CartSk,tmp_CartSk_edgeID,tmp_CartRel);
	
	vector<int> tmp_component(num_vertices(tmp_CartRel));
	int numero = connected_components(tmp_CartRel, &tmp_component[0]);
	//myEdgeW w = get(edge_weight, tmpG);
	//myEdgeID edge_id=get(edge_index,tmp_CartSk);
	/*for(tie(ei,ei_end) = edges(tmpG);ei!=ei_end;ei++){
		if(get(w,*ei) == 1){
			int id = get(edge_id, *ei);
			cout << endl << *ei << "=id" << id << " in comp " << tmp_component[id];
		}
	}
	for(tie(ei,ei_end) = edges(tmp_CartSk);ei!=ei_end;ei++){
			int id = get(edge_id, *ei);
			cout << endl << *ei << "=id " << id << " in comp " << tmp_component[id];
	}*/
	
	
	vector< vector<int> > tmp_coord_vert;
	init_coordinates(tmp_CartSk, tmp_CartRel, tmp_coord_vert, false);
	
	vector<myGraph> F;
	init_factor_corresponding_to_coordinates(tmp_coord_vert, tmpG, 2, F, false);
	
	//if(num_edges(G) != num_edges(F[0])*num_vertices(F[1]) + num_edges(F[1])*num_vertices(F[0])+num_edges(F[0])*num_edges(F[1])
	//	|| num_vertices(G)!=num_vertices(F[0])*num_vertices(F[1])) return false;
	
	
	/**### position1*/
	
	vector<myGraph> bothFactors;
	bothFactors.push_back(F[0]);bothFactors.push_back(F[1]);
	/***  if isom=true, than save H_1 as PrimeFactor, i.e. save elements of S, for combining later 9:54 AM 6/6/2009*/
	
	if(test_isomorphism(tmpG,tmp_coord_vert,tmp_component,numero, bothFactors, tmp_CartSk, tmp_CartRel)){
		return true;
	}
	else return false;
	
}

void combineFactors(myGraph& CartRel,/*const*/ myGraph& G, vector< set<int> >& S, 
					vector<int>& component, vector< vector<int> >& coord_vert,
					myGraph& CartSk, myGraph& CartSkEdgeID, vector<myGraph>& F){

	 

	 myGraph tmp_CartRel;
	 clear_graph(tmp_CartRel);
	 copy_graph(CartRel,tmp_CartRel);
	
	/**first new RelGraph, by connecting components induces by set S and set I_minus_S*/
	eIt ei, ei_end;
	
	vIt vi,vi_end, vi2, vi2_end;
	set<int>::iterator its, S_begin;
	
	for(unsigned int i = 0; i<S.size();i++){
	
		S_begin = S[i].begin();
		its=S[i].begin(); 
	
		if(S_begin!=S[i].end() && S[i].size()>1){
			++its;
		  //  cout << endl << "1." << *S_begin;
			for(tie(vi,vi_end) = vertices(CartRel); vi!=vi_end; ++vi){
				if(component[*vi] == *S_begin){
					while(its!=S[i].end()){
			//		 cout << endl << "2." << *its;
						for(tie(vi2,vi2_end) = vertices(CartRel); vi2!=vi2_end; ++vi2)
								if(component[*vi2] == *its)
									if(!edge(*vi,*vi2,tmp_CartRel).second){
										add_edge(*vi,*vi2,tmp_CartRel);
									}
						++its;
					}
				
					break;			
				}
			}	
		}
	}
	
	//######################################
	
	/**after new RelGraph, by connecting components induces by set S and set I_minus_S
	    compute new CartSk etc.pp.*/
	
	clear_graph(CartRel);
	copy_graph(tmp_CartRel,CartRel);
	
	/**after new RelGraph, by connecting components induces by set S and set I_minus_S
	    compute new CartSk etc.pp.*/
	/**also newRelGraph is new initialisert */


	
	init_newCartSk(G,CartSk,CartSkEdgeID,CartRel);


	
	component.clear();
	component.resize(num_vertices(CartRel));

	int numero = connected_components(CartRel, &component[0]);
	
	
	//vector<int>	compCartSk;
	//compCartSk.resize(num_vertices(CartSk));

	//int numCompCartSk = connected_components(CartSk, &compCartSk[0]);
	
	//if(numCompCartSk == 1){
	init_coordinates(CartSk, CartRel, coord_vert, false);
	init_factor_corresponding_to_coordinates(coord_vert, G, numero, F, false);
	//}
	//else{
	//	F.clear();
	//	F.push_back(G);		
	//	F.push_back(G);
	//}
}


void init_IminusS(set<int>& I,set<int>& S,set<int>& I_minus_S){
			set_difference(I.begin(), I.end(),
						   S.begin(), S.end(),
						   inserter(I_minus_S, I_minus_S.begin()));
}

void Subsets(int L, set<int>& J, vector<int>& elements_of_J, vector< set< int > >& Subsets_Of_J, 
		vector< set< int > >& length_L_subsets, vector<int>& bin){
//void Subsets(){
	
	elements_of_J.clear();
	bin.clear();
	Subsets_Of_J.clear();
	length_L_subsets.clear();
	
	set<int>::iterator its;
	for(its=J.begin();its!=J.end();++its){
		elements_of_J.push_back(*its);
		bin.push_back(0);
	}
	//for(int i=0;i<numFactors;i++)
	init_ALLSubsets(J.size(), bin, elements_of_J, Subsets_Of_J);
	//showMulti(Subsets_Of_J);
	//set< set<int> > Pow_J;Pow_J.clear();
	//generate_AllSubsets(Pow_J, J );
	//showMultiSet(Pow_J);
	//init_Sets_of_given_lenght(L, Pow_J, length_L_subsets );	
	init_Sets_of_given_lenght(L, Subsets_Of_J, length_L_subsets );	
		
}

int check_and_combineFactors(myGraph& G, myGraph& CartSk, myGraph& CartSkEdgeID, myGraph& CartRelGraph, 
							 vector< vector<int> >& coord_vert, vector<int>& component, int& numFactors,
							 vector<myGraph>& new_factors){
	/**loop due to isomorphismTest in NICE-paper**/
	
	numFactors = connected_components(CartRelGraph, &component[0]);

	if(numFactors == 1){ 
		return 0;
	}
	
	
	new_factors.clear();
	set<int> S,I,J;
	
	for(int i=0; i<numFactors; ++i){
		I.insert(i);
		J.insert(i);
	}
	
	//cout << endl << "J="; showSet(J);
	vector< set<int> >saveS;
	for(int L=1; L<=numFactors; ++L){
		
		if(L>int(J.size())) break;
		vector<int> elements_of_J;
		vector< set< int > > Subsets_Of_J;
		vector< set< int > > length_L_subsets;
		vector<int> bin;//bin.clear();
		
		
		Subsets(L, J,elements_of_J,Subsets_Of_J,length_L_subsets,bin);
		
		/**for eachS subset J of with |S|=k   HERE::  subset of J muss jedesmal neu initialisiert werden */
		
		for(unsigned int j = 0; j<length_L_subsets.size(); j++){
			
			S.clear();
			
			S = length_L_subsets[j];
			set<int> I_minus_S; I_minus_S.clear();
			init_IminusS(I,S,I_minus_S);
			
			
			
			if(!S.empty() && !I_minus_S.empty()){
				//cout << endl << "S" << j << " = "; showSet(S);
				//cout << endl << "IminusS  = "; showSet(I_minus_S);cout << endl;
				if(init_and_test_isom_H1_H2(CartRelGraph,G, S, I_minus_S,component, 
											coord_vert, CartSk, CartSkEdgeID)){

					//cout << endl << "############################";
					//cout << endl << "S" << j << " = "; showSet(S);
					//cout << endl << "IminusS  = "; showSet(I_minus_S);cout << endl;
					//cout << "saveS";
					saveS.push_back(S);
					set<int> oldJ;oldJ.clear();
					oldJ=J;J.clear();
					init_IminusS(oldJ,S,J);
					Subsets(L, J,elements_of_J,Subsets_Of_J,length_L_subsets,bin);
					j = 0;
					
				}
			}
		}
			
	}
	
	if(!J.empty()) saveS.push_back(J);
		
	/*cout << endl << "Factors to combine: ";
	for(unsigned int i = 0; i<saveS.size(); i++){
		cout << endl << i << " : "; showSet(saveS[i]);
	}*/
	
	
	
	
	combineFactors(CartRelGraph, G,saveS, component, coord_vert,
				   CartSk, CartSkEdgeID, new_factors);


	return 0;
	
}

	
